(function() {

var VERSION_URL = 'https://www.soft8soft.com/verge3d-' + PACKAGE.toLowerCase() + '-latest.json?' + Date.now();

function parseVersion(text) {
    version = [0, 0, 0, 0];

    dotSplit = text.split('.');
    preSplit = dotSplit[2].split('pre');

    version[0] = +dotSplit[0];
    version[1] = +dotSplit[1];
    version[2] = +preSplit[0];

    // consider stable release more recent than pre release
    if (preSplit.length > 1)
        version[3] = +preSplit[1];
    else
        version[3] = 1000;

    return version;
}

/**
 *  1 v1 > v2
 *  0 v1 = v2
 * -1 v1 < v2
 */
function compareVersions(v1, v2) {

    for (var i = 0; i < 4; i++) {
        var sign = Math.sign(v1[i] - v2[i]);
        if (sign)
            return sign;
    }

    return 0;
}

function checkForUpdates() {
    var versionElem = document.getElementsByClassName('version')[0];

    if (versionElem) {

        function reqListener() {
            if (this.status == 200) {
                var info = JSON.parse(this.responseText);

                var v1 = parseVersion(info.version);
                var v2 = parseVersion(VERSION);

                if (compareVersions(v1, v2) > 0) {
                    var newVerMsg = ' - <a href="' + info.release_notes +
                            '" target="_blank" title="Check out the new cool features right away!">update ' + info.version + ' available!</a>';
                    versionElem.innerHTML += newVerMsg;
                }
            }
        }

        var req = new XMLHttpRequest();
        req.addEventListener('load', reqListener);
        req.open('GET', VERSION_URL);
        req.send();
    }
}

function init() {
    if (CHECK_FOR_UPDATES)
        window.addEventListener('load', checkForUpdates);
}

init();

})(); // end of closure


