#!/usr/bin/env python

import os, platform, sys

SUPPORTED_MAYA_VERSIONS = [2017, 2018, 2019, 2020]

def registerPlugin(conf):

    baseDir = os.path.dirname(os.path.abspath(__file__))
    pluginPath = os.path.normpath(os.path.join(baseDir, '..'))

    with open(conf, 'a', encoding='utf8') as f:
        f.write('MAYA_PLUG_IN_PATH=' + pluginPath + '\n')

def traverseMayaDirs():
    system = platform.system()

    if system == 'Windows':
        mayaDir = os.path.expandvars(r'%USERPROFILE%\Documents\maya')
    elif system == 'Darwin':
        mayaDir = os.path.expandvars(r'$HOME/Library/Preferences/Autodesk/maya')
    else:
        mayaDir = os.path.expandvars(r'$HOME/maya')

    if os.path.exists(mayaDir):
        for mayaVer in SUPPORTED_MAYA_VERSIONS:
            maxVerDir = os.path.join(mayaDir, str(mayaVer))
            envConf = os.path.join(maxVerDir, 'Maya.env')
            if os.path.exists(envConf):
                print('Registering plugin path for Maya {}'.format(mayaVer))
                registerPlugin(envConf)

if __name__ == '__main__':
    traverseMayaDirs()
