const fs = require('fs');
const path = require('path');

function p(relPath) {
    return path.join(__dirname, relPath);
}

export default {
    entry: p('src/main.js'),
    targets: [
        {
            format: 'iife',
            dest: p('puzzles.min.js'),
            moduleName: 'v3d.PE',
            globals: {
                v3d: 'this.v3d',
                blockly: 'Blockly',
                goog: 'goog',
                ResizeSensor: 'ResizeSensor'
            }
        }
    ],
    banner() {
        return [
            p('src/blockly/blockly_compressed.js'),
            p('src/blockly/blockly_en.js'),
            p('src/blockly/blocks_compressed.js'),
            p('src/blockly/javascript_compressed.js'),
            p('src/misc/ResizeSensor.js')
        ].reduce((accum, filepath) => {
            return `${accum}\n${fs.readFileSync(filepath, 'utf8')}`;
        }, '');
    },
    footer() {
        return `// backwards compatibility for old player projects\nwindow.PE = this.v3d.PE;`;
    },
    external: ['v3d', 'blockly', 'goog', 'ResizeSensor'],
    onwarn(warning) {
        if (/Use of eval is strongly discouraged/.test(warning)) {
            return;
        }

        console.warn(warning.toString());
    },
    watch: { chokidar: { usePolling: true } }
}
