import { Clock } from '../core/Clock.js';
import { Color } from '../math/Color.js';

import { GearImage } from '../media.js';

function Preloader() {}

Object.assign(Preloader.prototype, {

    onUpdate: function(percentage) {},

    onFinish: function() {}

});


/**
 * parameters = {
 *  container: <HTMLElement|string>
 *  imageRotationSpeed: <float>
 * }
 */
function SimplePreloader(parameters) {

    Preloader.call(this);

    this.imageRotationSpeed = 200;

    for (var key in parameters) {
        var value = parameters[key];
        switch (key) {
        case 'container':
            if (value === undefined)
                this.container = document.body;
            else if (value instanceof HTMLElement)
                this.container = value;
            else
                this.container = document.getElementById(value);
            break;
        case 'imageRotationSpeed':
            this.imageRotationSpeed = value;
            break;
        }
    }

    this.text = document.createElement('div');
    this.text.setAttribute('class', 'v3d-simple-preloader-text');
    this.text.innerHTML = '0%';

    this.gear = document.createElement('div');
    this.gear.setAttribute('class', 'v3d-simple-preloader-gear');

    this.gearCont = document.createElement('div');
    this.gearCont.setAttribute('id', 'v3d_preloader_container');
    this.gearCont.setAttribute('class', 'v3d-simple-preloader-gear-container');

    this.background = document.createElement('div');
    this.background.setAttribute('class', 'v3d-simple-preloader-background');

    this.background.appendChild(this.gearCont);

    this.gearCont.appendChild(this.gear);
    this.gearCont.appendChild(this.text);

    this.container.appendChild(this.background);

    // assign default Data URL encoded gear image

    if (getComputedStyle(this.gear).backgroundImage == 'none')
        this.gear.setAttribute('style', 'background-image: url("' + GearImage + '");');

    this.clock = new Clock();

    var scope = this;

    var animCb = function() {
        if (scope.imageRotationSpeed > 0)
            requestAnimationFrame(animCb);

        var angle = scope.clock.getElapsedTime() * scope.imageRotationSpeed;
        scope.gear.style.transform = "rotate(" + angle + "deg)";
    }

    animCb();
}

SimplePreloader.prototype = Object.assign(Object.create(Preloader.prototype), {

    constructor: SimplePreloader,

    onUpdate: function(percentage) {
        percentage = Math.round(percentage);
        this.text.innerHTML = percentage + '%';
    },

    onFinish: function() {
        this.container.removeChild(this.background);
        this.imageRotationSpeed = 0;
    }

});

export { Preloader, SimplePreloader };
